/*
    Copyright 2006 Kevin Ottens <ervin@kde.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) version 3, or any
    later version accepted by the membership of KDE e.V. (or its
    successor approved by the membership of KDE e.V.), which shall
    act as a proxy defined in Section 6 of version 3 of the license.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library. If not, see <http://www.gnu.org/licenses/>.
*/

#include "wminetworkinterface.h"

#include "wmidevice.h"

#include <QtCore/QStringList>

using namespace Solid::Backends::Wmi;

NetworkInterface::NetworkInterface(WmiDevice *device)
    : DeviceInterface(device)
{

}

NetworkInterface::~NetworkInterface()
{

}

QString NetworkInterface::ifaceName() const
{
    return m_device->property("net.interface").toString();
}

bool NetworkInterface::isWireless() const
{
    QStringList capabilities = m_device->property("info.capabilities").toStringList();

    return capabilities.contains("net.80211");
}

QString NetworkInterface::hwAddress() const
{
    return m_device->property("net.address").toString();
}

qulonglong NetworkInterface::macAddress() const
{
    if (m_device->propertyExists("net.80211.mac_address"))
    {
        return m_device->property("net.80211.mac_address").toULongLong();
    }
    else
    {
        return m_device->property("net.80203.mac_address").toULongLong();
    }
}

#include "backends/wmi/wminetworkinterface.moc"
